@@ -52,6 +52,7 @@ pub use observable_comp::*;
 use crate::ops::default_if_empty::DefaultIfEmptyOp;
 use ops::{
   box_it::{BoxOp, IntoBox},
+  buffer::{BufferWithCountOp, BufferWithCountOrTimerOp, BufferWithTimeOp},
   contains::ContainsOp,
   debounce::DebounceOp,
   delay::DelayOp,
@@ -60,6 +61,7 @@ use ops::{
   filter_map::FilterMapOp,
   finalize::FinalizeOp,
   flatten::FlattenOp,
+  group_by::GroupByOp,
   last::LastOp,
   map::MapOp,
   map_to::MapToOp,
@@ -253,6 +255,50 @@ pub trait Observable: Sized {
     }
   }
 
+  /// Groups items emited by the source Observable into Observables.
+  /// Each emited Observable emits items matching the key returned
+  /// by the discriminator function.
+  ///
+  /// # Example
+  ///
+  /// ```
+  /// use rxrust::prelude::*;
+  ///
+  /// #[derive(Clone)]
+  /// struct Person {
+  ///   name: String,
+  ///   age: u32,
+  /// }
+  ///
+  /// observable::from_iter([
+  ///   Person{ name: String::from("John"), age: 26 },
+  ///   Person{ name: String::from("Anne"), age: 28 },
+  ///   Person{ name: String::from("Gregory"), age: 24 },
+  ///   Person{ name: String::from("Alice"), age: 28 },
+  /// ])
+  /// .group_by(|person: &Person| person.age)
+  /// .subscribe(|group| {
+  ///   group
+  ///   .reduce(|acc, person| format!("{} {}", acc, person.name))
+  ///   .subscribe(|result| println!("{}", result));
+  /// });
+  ///
+  /// // Prints:
+  /// //  John
+  /// //  Anne Alice
+  /// //  Gregory
+  /// ```
+  #[inline]
+  fn group_by<D, Item, Key>(self, discr: D) -> GroupByOp<Self, D>
+  where
+    D: FnMut(&Item) -> Key,
+  {
+    GroupByOp {
+      source: self,
+      discr,
+    }
+  }
+
   /// Creates a new stream which calls a closure on each element and uses
   /// its return as the value.
   #[inline]
@@ -1193,6 +1239,127 @@ pub trait Observable: Sized {
       default_value,
     }
   }
+
+  /// Buffers emitted values of type T in a Vec<T> and
+  /// emits that Vec<T> as soon as the buffer's size equals
+  /// the given count.
+  /// On complete, if the buffer is not empty,
+  /// it will be emitted.
+  /// On error, the buffer will be discarded.
+  ///
+  /// The operator never returns an empty buffer.
+  ///
+  /// #Example
+  /// ```
+  /// use rxrust::prelude::*;
+  ///
+  /// observable::from_iter(0..6)
+  ///   .buffer_with_count(3)
+  ///   .subscribe(|vec| println!("{:?}", vec));
+  ///
+  /// // Prints:
+  /// // [0, 1, 2]
+  /// // [3, 4, 5]
+  /// ```
+  #[inline]
+  fn buffer_with_count(self, count: usize) -> BufferWithCountOp<Self> {
+    BufferWithCountOp {
+      source: self,
+      count,
+    }
+  }
+
+  /// Buffers emitted values of type T in a Vec<T> and
+  /// emits that Vec<T> periodically.
+  ///
+  /// On complete, if the buffer is not empty,
+  /// it will be emitted.
+  /// On error, the buffer will be discarded.
+  ///
+  /// The operator never returns an empty buffer.
+  ///
+  /// #Example
+  /// ```
+  /// use rxrust::prelude::*;
+  /// use std::time::Duration;
+  /// use futures::executor::ThreadPool;
+  ///
+  /// let pool = ThreadPool::new().unwrap();
+  ///
+  /// observable::create(|mut subscriber| {
+  ///   subscriber.next(0);
+  ///   subscriber.next(1);
+  ///   std::thread::sleep(Duration::from_millis(100));
+  ///   subscriber.next(2);
+  ///   subscriber.next(3);
+  ///   subscriber.complete();
+  /// })
+  ///   .buffer_with_time(Duration::from_millis(50), pool)
+  ///   .into_shared()
+  ///   .subscribe(|vec| println!("{:?}", vec));
+  ///
+  /// // Prints:
+  /// // [0, 1]
+  /// // [2, 3]
+  /// ```
+  #[inline]
+  fn buffer_with_time<S>(
+    self,
+    time: Duration,
+    scheduler: S,
+  ) -> BufferWithTimeOp<Self, S> {
+    BufferWithTimeOp {
+      source: self,
+      time,
+      scheduler,
+    }
+  }
+
+  /// Buffers emitted values of type T in a Vec<T> and
+  /// emits that Vec<T> either if the buffer's size equals count, or
+  /// periodically. This operator combines the functionality of
+  /// buffer_with_count and buffer_with_time.
+  ///
+  /// #Example
+  /// ```
+  /// use rxrust::prelude::*;
+  /// use std::time::Duration;
+  /// use futures::executor::ThreadPool;
+  ///
+  /// let pool = ThreadPool::new().unwrap();
+  ///
+  /// observable::create(|mut subscriber| {
+  ///   subscriber.next(0);
+  ///   subscriber.next(1);
+  ///   subscriber.next(2);
+  ///   std::thread::sleep(Duration::from_millis(100));
+  ///   subscriber.next(3);
+  ///   subscriber.next(4);
+  ///   subscriber.complete();
+  /// })
+  ///   .buffer_with_count_and_time(2, Duration::from_millis(50), pool)
+  ///   .into_shared()
+  ///   .subscribe(|vec| println!("{:?}", vec));
+  ///
+  /// // Prints:
+  /// // [0, 1]
+  /// // [2]
+  /// // [3, 4]
+  /// ```
+  #[inline]
+  fn buffer_with_count_and_time<S>(
+    self,
+    count: usize,
+    time: Duration,
+    scheduler: S,
+  ) -> BufferWithCountOrTimerOp<Self, S> {
+    BufferWithCountOrTimerOp {
+      source: self,
+      count,
+      time,
+      scheduler,
+    }
+  }
 }
 
 pub trait LocalObservable<'a>: Observable {
